@@ -3,7 +3,7 @@
 #
 # Author: Slaven Rezic
 #
-# Copyright (C) 2009,2010,2012,2017 Slaven Rezic. All rights reserved.
+# Copyright (C) 2009 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -13,11 +13,9 @@
 
 package Algorithm::GooglePolylineEncoding;
 
-use 5.006; # sprintf("%b")
-
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.05_50';
+$VERSION = '0.01';
 
 sub encode_number {
 #   1. Take the initial signed value:
@@ -72,13 +70,14 @@ sub encode_number {
 sub encode_polyline {
     my(@path) = @_;
     my @res;
-    my($curr_lat_e5,$curr_lon_e5) = (0,0);
+    my($curr_lat,$curr_lon) = do { my $first = shift @path; ($first->{lat}, $first->{lon}) };
+    push @res, encode_number($curr_lat), encode_number($curr_lon);
     for my $lat_lon (@path) {
-	my($lat_e5,$lon_e5) = map { sprintf("%.0f", $_*1e5) } ($lat_lon->{lat}, $lat_lon->{lon});
-        my $deltay = ($lat_e5 - $curr_lat_e5) / 1e5;
-        my $deltax = ($lon_e5 - $curr_lon_e5) / 1e5;
+        my($lat,$lon) = ($lat_lon->{lat}, $lat_lon->{lon});
+        my $deltay = $lat - $curr_lat;
+        my $deltax = $lon - $curr_lon;
         push @res, encode_number($deltay), encode_number($deltax);
-        ($curr_lat_e5,$curr_lon_e5) = ($lat_e5,$lon_e5);
+        ($curr_lat,$curr_lon) = ($lat,$lon);
     }
     join '', @res;
 }
@@ -89,15 +88,7 @@ sub encode_level {
     my $number = shift;
 #   2. Convert the decimal value to a binary value:
 #      10101110
-    my $bin;
-    if ($number > ~0) {
-	# sprintf '%b' works only for integers
-	require Math::BigInt;
-	$bin = Math::BigInt->new($number)->as_bin;
-	$bin =~ s{^0b}{};
-    } else {
-	$bin = sprintf '%b', $number;
-    }
+    my $bin = sprintf '%b', $number;
 #   3. Break the binary value out into 5-bit chunks (starting from the right hand side):
 #      101 01110
     $bin = '0'x(5-length($bin)%5) . $bin if length($bin)%5 != 0; # pad
@@ -122,71 +113,6 @@ sub encode_level {
     join '', @chunks;
 }
 
-# Translated this php script
-# <http://unitstep.net/blog/2008/08/02/decoding-google-maps-encoded-polylines-using-php/>
-# to perl
-sub decode_polyline {
-    my($encoded) = @_;
-
-    my $length = length $encoded;
-    my $index = 0;
-    my @points;
-    my $lat = 0;
-    my $lng = 0;
-
-    while ($index < $length) {
-	# The encoded polyline consists of a latitude value followed
-	# by a longitude value. They should always come in pairs. Read
-	# the latitude value first.
-	for my $val (\$lat, \$lng) {
-	    my $shift = 0;
-	    my $result = 0;
-	    # Temporary variable to hold each ASCII byte.
-	    my $b;
-	    do {
-		# The `ord(substr($encoded, $index++))` statement returns
-		# the ASCII code for the character at $index. Subtract 63
-		# to get the original value. (63 was added to ensure
-		# proper ASCII characters are displayed in the encoded
-		# polyline string, which is `human` readable)
-		$b = ord(substr($encoded, $index++, 1)) - 63;
-
-		# AND the bits of the byte with 0x1f to get the original
-		# 5-bit `chunk. Then left shift the bits by the required
-		# amount, which increases by 5 bits each time. OR the
-		# value into $results, which sums up the individual 5-bit
-		# chunks into the original value. Since the 5-bit chunks
-		# were reversed in order during encoding, reading them in
-		# this way ensures proper summation.
-		$result |= ($b & 0x1f) << $shift;
-		$shift += 5;
-	    }
-		# Continue while the read byte is >= 0x20 since the last
-		# `chunk` was not OR'd with 0x20 during the conversion
-		# process. (Signals the end)
-		while ($b >= 0x20);
-
-	    use integer; # see last paragraph of "Integer Arithmetic" in perlop.pod
-
-	    # Check if negative, and convert. (All negative values have the last bit
-	    # set)
-	    my $dtmp = (($result & 1) ? ~($result >> 1) : ($result >> 1));
-
-	    # Compute actual latitude (resp. longitude) since value is
-	    # offset from previous value.
-	    $$val += $dtmp;
-	}
-
-	# The actual latitude and longitude values were multiplied by
-	# 1e5 before encoding so that they could be converted to a 32-bit
-	# integer representation. (With a decimal accuracy of 5 places)
-	# Convert back to original values.
-	push @points, {lat => $lat * 1e-5, lon => $lng * 1e-5};
-    }
-
-    @points;
-}
-
 1;
 
 __END__
@@ -206,7 +132,7 @@ Algorithm::GooglePolylineEncoding - Google's Encoded Polyline Algorithm Format
 B<Algorithm::GooglePolylineEncoding> implements the encoded polyline
 algorithm format which is used in some parts of the Google Maps API.
 The algorithm is described in
-L<https://developers.google.com/maps/documentation/utilities/polylinealgorithm>.
+L<http://code.google.com/intl/en/apis/maps/documentation/polylinealgorithm.html>.
 
 This module is a light-weight version of
 L<Geo::Google::PolylineEncoder>, essentially just doing the encoding
@@ -220,9 +146,8 @@ dependencies.
 =item encode_polyline(@polyline)
 
 Take an array of C<< {lat => ..., lon => ...} >> hashrefs and return
-an encoded polyline string. Latitudes and longitudes should be
-expressed as decimal degrees (DD;
-L<http://en.wikipedia.org/wiki/Decimal_degrees>).
+an encoded polyline. Latitudes and longitudes should be expressed as
+decimal degrees (DD; L<http://en.wikipedia.org/wiki/Decimal_degrees>).
 
 =item encode_level($level)
 
@@ -233,20 +158,19 @@ Return an encoded level.
 Return just an encoded number (which may be a single longitude, or
 latitude, or delta).
 
-=item decode_polyline($encoded)
-
-Take an encoded polyline string and return a list of C<< {lat => ...,
-lon => ...} >> hashrefs.
-
 =back
 
+=head1 TODO
+
+Implement decode_polyline.
+
 =head1 AUTHOR
 
 Slaven Rezic <srezic@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2009,2010,2012,2017 Slaven Rezic. All rights reserved.
+Copyright (c) 2009 Slaven Rezic. All rights reserved.
 This module is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
